"""
    These are the sub-commands for a top level object added to the CLI group in 'cli.py'.
    The commands and options are implemented here and the logic behind them resides in the corresponding behavior file.
"""

from cli import pyke, behavior
import click

@click.group(help='Experience related commands.')
def experience():
  pass

@click.command('ls', help="List the experiences related to the selected profile.")
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command.', autocompletion=pyke.auth.get_profile_names)
@click.option('--format', '-f', default='{id} {label} {name} {createdAt}', help='The --format option takes the column name of the returned table wrapped in \
{} and returns only that column. It is not compatible with --json flag.')
@click.option('--filter', default='')
@click.option('--page', default=1)
@click.option('--pagesize', default=100)
@click.option('--json', is_flag=True, help='Return raw json from the platform.')
@pyke.auth.profile_context_required
def list(**kwargs):
  behavior.ExperienceBehavior(**kwargs).list()

@click.command('export', help='Exports an experience as a JSON file')
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command.', autocompletion=pyke.auth.get_profile_names)
@click.option('--label', '-l', help='The label of the experience being exported. This is the experience "Name" in the UI.')
@click.option('--name', '-n', default='', help='The autogenerated name (aka namespace) of the experience being exported. This is not the experience "Name" in the UI.')
@click.option('--export-file', '-path', help='The location to save the export file. Not supported if --json is specified.')
@click.option('--json', is_flag=True, help='Return raw json from the platform')
@pyke.auth.profile_context_required
def export(**kwargs):
  behavior.ExperienceBehavior(**kwargs).export()

@click.command('import', help='Imports an experience via a JSON file')
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command.', autocompletion=pyke.auth.get_profile_names)
@click.option('--label', '-l', prompt=True, help='The label for the new experience. This aligns with "Name" in the UI.')
@click.option('--description', '-d', help='The description for the new experience')
@click.option('--collection-id', '-ci', default='', help='The experience collection ID for the new experience')
@click.option('--device', default=None, help='The preview device the experience is displayed in while in the studio')
@click.option('--collection-name', '-cn', help='The experience collection name for the new experience')
@click.option('--activation-code', '-ac', prompt=True, help='The activation code for the new experience')
@click.option('--template', '-t', is_flag=True, help='Indicates the new experience should be a template')
@click.option('--redirect-url', '-ru', default='', help='The redirect url for the new experience')
@click.option('--import-file', '-path', prompt=True, help='The location of the experience file to import')
@click.option('--json', is_flag=True, help='Return raw json from the platform')
@pyke.auth.profile_context_required
def import_(**kwargs):
  behavior.ExperienceBehavior(**kwargs).import_()

experience.add_command(list)
experience.add_command(export)
experience.add_command(import_)



from cli import pyke, behavior, commands
import click

@click.command('logs', help='Tail logs for an editable container.')
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command.', autocompletion=pyke.auth.get_profile_names)
@click.option('--container', '-c', prompt=True, help='The ID or the url-ref of the container the editable version belongs to.')
@click.option('--version', '-v', prompt=True, help='The version ID or the version-number')
@pyke.auth.profile_context_required
def tail(container, version, profile):
  behavior.ContainerVersionBehavior(profile=profile).tail_logs(container, version)


@click.command(help='This force updates your docker service. This can recover a process in a bad state.')
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command.', autocompletion=pyke.auth.get_profile_names)
@click.option('--container', '-c', prompt=True, help='The ID or the url-ref of the container the editable version belongs to.')
@click.option('--version', '-v', prompt=True, help='The version ID or the version-number')
@pyke.auth.profile_context_required
def force(container, version, profile):
  behavior.ContainerVersionBehavior(profile=profile).force_update(container, version)

@click.command('download', help='Download a zip file that contains this containers application source code')
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command.', autocompletion=pyke.auth.get_profile_names)
@click.option('--container', '-c', prompt=True, help='The ID or the url-ref of the container the editable version belongs to.')
@click.option('--version', '-v', prompt=True, help='The version ID or the version-number')
@click.option('--path', type=str, help='Path to save zip file')
@pyke.auth.profile_context_required
def download_app_zip(container, version, profile, path):
  behavior.ContainerVersionBehavior(profile=profile).download_app_zip(container, version, path)


commands.microservice_version.add_command(download_app_zip)
commands.widget_version.add_command(download_app_zip)

commands.microservice_version.add_command(force)
commands.widget_version.add_command(force)

commands.microservice_version.add_command(tail)
commands.widget_version.add_command(tail)



from cli import pyke, commands
from .version_cmd import version
import click

"""
    This is the top level of the CLI command tree. The CLI group is created and the
    subcommands are imported from the commands dir and added to the group. This allows
    these commands to be run with 'luma <command>'.
"""

@click.group(help='This CLI will allow you to interact with the Lumavate platform from your terminal. For setup instructions, look in github. \
Each command below has subcommands. Pass the --help flag to those commands for more information on how to use them.')
def cli():
    pass

cli.add_command(commands.env)
cli.add_command(commands.profile)
cli.add_command(commands.organization)
cli.add_command(commands.widget)
cli.add_command(commands.widget_version)
cli.add_command(commands.component_set)
cli.add_command(commands.component_set_version)
cli.add_command(commands.microservice)
cli.add_command(commands.microservice_version)
cli.add_command(version)
cli.add_command(commands.api)
cli.add_command(commands.experience)
cli.add_command(commands.experience_collection)

if __name__ == '__main__':
    cli(help='')



"""
    This behavior file contains the logic for a subset of commands. Logic specific to
    commands should be implamented in corresponding behavior files.
"""

from pathlib import Path
from cli import pyke
import requests
import click
import time
import json
import sys
import os

class ComponentSetVersionBehavior:
    def __init__(self, profile=None, component_set=None, format=None, filter=None, page=None, pagesize=None, json=False):
        self.util = pyke.Util(profile=profile)
        if profile is not None:
            self.context = pyke.auth.load_cache()
        self.object_type = 'component-set'
        self.json = json
        self.profile = profile
        self.component_set = component_set
        self.format = format
        self.filter = filter
        self.page = page
        self.pagesize = pagesize

    def list(self, component_set):
      component_set_id = self.util.lookup_object_id(self.object_type, component_set)

      click.echo(component_set_id)
      data = {'filter': self.filter, 'page': self.page, 'pagesize': self.pagesize}

      titles = {
          'directIncludes': '# Inc',
          'directCssIncludes': '# Css Inc',
          'expand__experiences': '# Exp',
          'expand__components': '# Comp',
      }

      handlers = {
          'directIncludes': lambda x: str(len(x)),
          'directCssIncludes': lambda x: str(len(x)),
          'expand__experiences': lambda x: str(len(x)),
          'expand__components': lambda x: str(len(x))
      }

      resp = self.util.cli_request('GET',
          self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions?expand=experiences&page={page}&pagesize={pagesize}&{filter}',
          {'component_set_id': component_set_id, **data}))

      if not self.json:
        self.util.print_table(resp['payload']['data'], self.format, titles=titles, handlers=handlers)
        self.util.print_record_count(resp)
      else:
        click.echo(json.dumps(resp))

    def list_components(self, component_set, version):
        component_set_id = self.util.lookup_object_id(self.object_type, component_set)
        version_id = self.util.get_version_id(self.object_type, component_set_id, version)

        resp = self.util.cli_request('GET',
            self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions/{version_id}?expand=components',\
            {'component_set_id': component_set_id, 'version_id': version_id}))

        click.echo(json.dumps(resp))

    def create(self, component_set, component_set_file_path, from_version, version_number,
        version_bump, css_includes, direct_includes, label):
        component_set_id = self.util.lookup_object_id(self.object_type, component_set)

        if from_version != '' and from_version is not None:
            try:
                from_version_id = int(from_version)
                version = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions/{from_version_id}',\
                    {'component_set_id': component_set_id, 'from_version_id': from_version_id} ))['payload']['data']
            except:
                version = self.util.get_version(self.object_type, component_set_id, from_version)

            if version is None:
                raise click.BadParameter(click.style("Component Set version not found", fg='red'), param_hint="--version-number")

            version[version_bump] = version[version_bump] + 1
            if version_bump in ['major', 'minor']:
                version['patch'] = 0
            if version_bump == 'major':
                version['minor'] = 0

        else:
            version = {}
            if label is None or label == '':
                label = input("Label: ")
                if label is None or label == '':
                    raise click.BadParameter(click.style("If you don't create from an old version then you must provide a --label", fg='red'), param_hint="--label")

            if version_number is None or version_number == '':
                version_number = input("Version Number: ")
                if version_number is None or version_number == '':
                    raise click.BadParameter(click.style("If you don't create from an old version then you must provide a --version-number in the format\
                        '<major: int>.<minor: int>.<patch: int>' ", fg='red'), param_hint="--version-number")

            major, minor, patch = self.util.parse_version_number(version_number)

            version['major'] = major
            version['minor'] = minor
            version['patch'] = patch

        if not self.json:
            click.echo('Image Size: {}'.format(self.util.get_size(component_set_file_path, image=False)))
            click.echo("Uploading Component Set Version to Lumavate")

        version['ephemeralKey'] = self.util.upload_ephemeral(component_set_file_path, 'application/zip')

        if label != '' and label is not None:
            version['label'] = label

        if css_includes:
            if 'directCssIncludes' not in version.keys():
                version['directCssIncludes'] = [x for x in css_includes]
            else:
                version['directCssIncludes'].extend([x for x in css_includes])

        if direct_includes:
            if 'directIncludes' not in version.keys():
                version['directIncludes'] = [x for x in direct_includes]
            else:
                version['directIncludes'].extend([x for x in direct_includes])

        resp = self.util.cli_request('POST',
            self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions',
            {'component_set_id': component_set_id}), data=json.dumps(version))

        if self.json:
            click.echo(json.dumps(resp))
            return

        handlers = {
          'directIncludes': lambda x: str(len(x)),
          'directCssIncludes': lambda x: str(len(x))
        }

        self.util.print_table([resp['payload']['data']], self.format, handlers=handlers)

    def update(self, component_set, version, label):
        component_set_id = self.util.lookup_object_id(self.object_type, component_set)
        version_id = self.util.get_version_id(self.object_type, component_set_id, version)

        post_data = {
          'componentSetId': component_set_id,
          'id': version_id,
          'label': label
        }

        resp = self.util.cli_request('PUT',
          self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions/{version_id}',
          {'component_set_id': component_set_id, 'version_id': version_id}), data=json.dumps(post_data))

        if self.json:
            click.echo(json.dumps(resp))
            return

        handlers = {
          'directIncludes': lambda x: str(len(x)),
          'directCssIncludes': lambda x: str(len(x))
        }

        self.util.print_table([resp['payload']['data']], self.format, handlers=handlers)

    def delete(self, component_set, version, version_mask):
        component_set_id = self.util.lookup_object_id(self.object_type, component_set)

        if version is None and version_mask is None:
            version = str(input("Version: "))
            if version is None or version == '':
                raise click.BadParameter(click.style('You must provide either a --version or a --version-mask', fg='red'))

        if version is not None:
            version_id = self.util.get_version_id(self.object_type, component_set_id, version)

            resp = self.util.cli_request('DELETE',
              self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions/{version_id}',
                {'component_set_id': component_set_id, 'version_id': version_id}))

            if self.json:
                click.echo(json.dumps(resp))
                return

            handlers = {
              'directIncludes': lambda x: str(len(x)),
              'directCssIncludes': lambda x: str(len(x))
            }

            self.util.print_table([resp['payload']['data']], self.format, handlers=handlers)

        else:
            versions = self.util.get_versions_from_mask(self.object_type, component_set_id, version_mask)
            if self.json:
                resp_list = {}
                resp_list['responses'] = []
                for v in versions:
                    resp = self.util.cli_request('DELETE',
                        self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions/{version_id}', {'component_set_id': component_set_id, 'version_id': v['id']}))
                    resp_list['responses'].append(resp)
                click.echo(json.dumps(resp_list))
                return

            with click.progressbar(versions, label='Deleting Versions') as bar:
                for v in bar:
                    resp = self.util.cli_request('DELETE',
                        self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions/{version_id}', {'component_set_id': component_set_id, 'version_id': v['id']}))

            handlers = {
                  'directIncludes': lambda x: str(len(x)),
                  'directCssIncludes': lambda x: str(len(x))
            }

            self.util.print_table(versions, self.format, handlers=handlers)



"""
    This behavior file contains the logic for a subset of commands. Logic specific to
    commands should be implamented in corresponding behavior files.
"""

from cli import pyke
import requests
import click
import json

class ExperienceCollectionBehavior:
  def __init__(self, **kwargs):
    profile = kwargs.get('profile')
    self.util = pyke.Util(profile=profile)
    if profile is not None:
      self.context = pyke.auth.load_cache()

    self.object_type = 'experience-collection'
    self.json = kwargs.get('json', False)
    self.profile = profile
    self.format = kwargs.get('format')
    self.filter = kwargs.get('filter')
    self.page = kwargs.get('page')
    self.pagesize = kwargs.get('pagesize')
    self.name = kwargs.get('name')

  def list(self):
    data = {'filter': self.filter, 'page': self.page, 'pagesize': self.pagesize}

    resp = self.util.cli_request('GET',
      self.util.build_url('{app}/iot/v1/experience-collections?page={page}&pagesize={pagesize}&{filter}', {**data} ))

    if self.json:
      click.echo(json.dumps(resp))
      return

    self.util.print_table(resp.get('payload', {}).get('data', {}), self.format)
    self.util.print_record_count(resp)


